#!/usr/bin/python
# -*- coding: utf8 -*-
# -*- Mode: Python; py-indent-offset: 4 -*-
# pylint: disable=C0103

"""Currentcost adpaters."""

from timeserieslogger.utils import INCORRECT_XML_MESSAGE
from timeserieslogger.utils import INCORRECT_CURRENTCOST_MISSING_TMPR
from timeserieslogger.utils import INCORRECT_CURRENTCOST_MISSING_WATTS
from timeserieslogger.db import get_devices_informations
from timeserieslogger.db import get_last_series, create_series
from datetime import datetime
from xml.etree import ElementTree
import json


def currentcost_kwh(energy_variable_id, power_variable_id, message):
    """Function that compute kwh consumption from watts

        :param energy_variable_id: Energy variable id.
        :type energy_variable_id: str.

        :param watts_variable_id: Power variable id.
        :type watts_variable_id: str.

        :param message: Message from RabbitMQ.
        :type message: dict.
    """
    site_id = message["siteID"]
    last_energy_series = get_last_series(site_id, energy_variable_id)
    last_power_series = get_last_series(site_id, power_variable_id)

    try:
        last_energy_value = last_energy_series[0]["value"]
        last_power_value = last_power_series[0]["value"]
        last_energy_date = datetime.strptime(
            last_energy_series[0]["date"],
            '%Y-%m-%dT%H:%M:%S.%f')
        actual_date = datetime.strptime(
            message["date"],
            '%Y-%m-%dT%H:%M:%S.%f')
        delta = actual_date - last_energy_date
        ws = last_power_value * delta.total_seconds()
        wh = ws / 3600
        kwh = wh / 1000
        last_energy_value += kwh
        return round(last_energy_value, 6)
    except IndexError:
        return 0.0


def currentcost_series(message):
    """Transform currentcost XML message into according series.

        :param message: Message from RabbitMQ.
        :type message: dict.
    """
    error = None
    series = []
    try:
        ch1_w = None
        ch2_w = None
        ch3_w = None
        xml = ElementTree.fromstring(message["message"])
        weather = xml.findtext("tmpr")
        hist = xml.findtext("hist")
        if hist is None:
            if weather is None:
                error = INCORRECT_CURRENTCOST_MISSING_TMPR % (
                    json.dumps(message))
            else:
                if xml.find("ch1") is not None:
                    ch1_w = xml.find("ch1").findtext("watts")
                if xml.find("ch2") is not None:
                    ch2_w = xml.find("ch2").findtext("watts")
                if xml.find("ch3") is not None:
                    ch3_w = xml.find("ch3").findtext("watts")
                if ch1_w is None and ch2_w is None and ch3_w is None:
                    error = INCORRECT_CURRENTCOST_MISSING_WATTS % (
                        json.dumps(message))
                else:
                    device = get_devices_informations(message["deviceID"])
                    for key in device['params']:
                        if key == 'ch1' and ch1_w is not None:
                            value = int(ch1_w)
                        elif key == 'ch1_kwh' and ch1_w is not None:
                            value = currentcost_kwh(
                                device['params'][key],
                                device['params']['ch1'],
                                message)
                        elif key == 'ch2' and ch2_w is not None:
                            value = int(ch2_w)
                        elif key == 'ch2_kwh' and ch2_w is not None:
                            value = currentcost_kwh(
                                device['params'][key],
                                device['params']['ch2'],
                                message)
                        elif key == 'ch3' and ch3_w is not None:
                            value = int(ch3_w)
                        elif key == 'ch3_kwh' and ch3_w is not None:
                            value = currentcost_kwh(
                                device['params'][key],
                                device['params']['ch3'],
                                message)
                        elif key == 'tmpr' and weather is not None:
                            value = weather
                        if value is None:
                            error = INCORRECT_CURRENTCOST_MISSING_WATTS % (
                                json.dumps(message))
                        else:
                            value = float(value)
                            new_series = create_series(
                                value,
                                device['params'][key],
                                message)
                            series.append(new_series)
    except ElementTree.ParseError:
        error = INCORRECT_XML_MESSAGE % message["message"]

    return error, series
